#!/usr/bin/env python
# -*- coding: utf-8 -*-
###########################################################################
#                                                                         #
#  Copyright (C) 2017  Rafal Kobel <rafalkobel@rafyco.pl>                 #
#                                                                         #
#  This program is free software: you can redistribute it and/or modify   #
#  it under the terms of the GNU General Public License as published by   #
#  the Free Software Foundation, either version 3 of the License, or      #
#  (at your option) any later version.                                    #
#                                                                         #
#  This program is distributed in the hope that it will be useful,        #
#  but WITHOUT ANY WARRANTY; without even the implied warranty of         #
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the           #
#  GNU General Public License for more details.                           #
#                                                                         #
#  You should have received a copy of the GNU General Public License      #
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.  #
#                                                                         #
###########################################################################
"""
Command line program to automatic downloading files.

Program checking file and download all file descripe in it. It's recomended to
add this file to crontab or call it manualy.

Example usage
=============

To invoke program type in your console::

    ytrss_deamon

or::

    python -m ytrss.daemon

for more option call program with flag C{--help}
"""

from __future__ import unicode_literals
from __future__ import print_function
import os
import sys
import logging
from argparse import ArgumentParser
from ytrss import get_version
from ytrss.core import UrlRememberer
from ytrss.core.locker import Locker, LockerError
from ytrss.core.downloader import Downloader
from ytrss.core.settings import YTSettings
from ytrss.core.settings import SettingException
try:
    import argcomplete
except ImportError:
    pass


def __option_args(argv=None):
    """
    Parsing argument for command line program.

    @param argv: Option parameters
    @type argv: list
    @return: parsed arguments
    """
    parser = ArgumentParser(description="Download all Youtube's movie "
                                        "to youtube path.",
                            prog='ytrss_daemon')
    parser.add_argument("-v", "--version", action='version',
                        version='%(prog)s {}'.format(get_version()))
    parser.add_argument("-c", "--conf", dest="configuration",
                        help="configuration file", default="", metavar="FILE")
    parser.add_argument("-l", "--log", dest="logLevel",
                        choices=['DEBUG', 'INFO', 'WARNING',
                                 'ERROR', 'CRITICAL'],
                        help="Set the logging level")
    try:
        argcomplete.autocomplete(parser)
    except NameError:
        pass
    options = parser.parse_args(argv)
    return options


class DaemonError(Exception):
    """ Daemon Exception. """
    pass


def download_all_movie(settings):
    """
    Download all movie saved in download_file.

    @param settings: Settings handle
    @type settings: L{YTSettings<ytrss.core.settings.YTSettings>}
    """
    logging.info("download movie from urls")
    locker = Locker('lock_ytdown')
    try:
        locker.lock()
    except LockerError:
        print("Program is running.")
        sys.exit(1)
    try:
        if not os.path.isfile(
                settings.download_file) and not os.path.isfile(
                settings.url_backup):
            raise DaemonError
        urls = UrlRememberer(settings.download_file)
        urls.read_backup(settings.url_backup)
        urls.delete_file()
        urls.save_as(settings.url_backup)

        error_file = UrlRememberer(settings.err_file)
        history_file = UrlRememberer(settings.history_file)

        for elem in urls.database:
            if not history_file.is_new(elem):
                print("URL {} cannot again download".format(elem))
                continue
            task = Downloader(settings, elem)
            if task.download():
                # finish ok
                print("finish ok")
                history_file.add_element(elem)
            else:
                # finish error
                print("finish error")
                error_file.add_element(elem)

        locker.unlock()

        os.remove(settings.url_backup)
    except KeyboardInterrupt as ex:
        locker.unlock()
        print("Keyboard Interrupt by user.")
        sys.exit(1)
    except DaemonError:
        locker.unlock()
        logging.debug("Cannot find url to download")
        sys.exit()
    except Exception as ex:
        locker.unlock()
        print("Unexpected Error: {}".format(ex))
        raise ex


# pylint: disable=R0915
def main(argv=None):
    """
    Main function for command line program.

    @param argv: Option parameters
    @type argv: list
    """
    options = __option_args(argv)
    logging.basicConfig(format='%(asctime)s - %(name)s - '
                               '%(levelname)s - %(message)s',
                        level=options.logLevel)
    logging.debug("Debug mode: Run")
    try:
        settings = YTSettings(options.configuration)
    except SettingException:
        print("Configuration file not exist.")
        sys.exit(1)
    download_all_movie(settings)


def daemon():
    """
    Daemon program for Linux.
    @todo: write daemon
    """
    print("Not implemented yet.")
    sys.exit(1)

if __name__ == "__main__":
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
###########################################################################
#                                                                         #
#  Copyright (C) 2017  Rafal Kobel <rafalkobel@rafyco.pl>                 #
#                                                                         #
#  This program is free software: you can redistribute it and/or modify   #
#  it under the terms of the GNU General Public License as published by   #
#  the Free Software Foundation, either version 3 of the License, or      #
#  (at your option) any later version.                                    #
#                                                                         #
#  This program is distributed in the hope that it will be useful,        #
#  but WITHOUT ANY WARRANTY; without even the implied warranty of         #
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the           #
#  GNU General Public License for more details.                           #
#                                                                         #
#  You should have received a copy of the GNU General Public License      #
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.  #
#                                                                         #
###########################################################################
"""
Testing module ytrss.

TestCase checking module files. It should be conform to PEP8.

@see: L{ytrss}
"""

from __future__ import unicode_literals
import unittest
import pep8
import ytrss


class TestYTRSSModule(unittest.TestCase):
    """ Module testsCase. """
    def test_pap8(self):
        """ Test that we conform to PEP8. """
        pep8_style = pep8.StyleGuide(paths=['--ignore=E501'])
        # Disable E501 code (line too long). It should be enabled after fixed.
        result = pep8_style.check_files(ytrss.__path__)
        self.assertEqual(result.total_errors, 0,
                         "Found code style errors (and warnings).")

if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
###########################################################################
#                                                                         #
#  Copyright (C) 2017  Rafal Kobel <rafalkobel@rafyco.pl>                 #
#                                                                         #
#  This program is free software: you can redistribute it and/or modify   #
#  it under the terms of the GNU General Public License as published by   #
#  the Free Software Foundation, either version 3 of the License, or      #
#  (at your option) any later version.                                    #
#                                                                         #
#  This program is distributed in the hope that it will be useful,        #
#  but WITHOUT ANY WARRANTY; without even the implied warranty of         #
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the           #
#  GNU General Public License for more details.                           #
#                                                                         #
#  You should have received a copy of the GNU General Public License      #
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.  #
#                                                                         #
###########################################################################
"""
Download mp3 file from YouTube using I{youtube_dl} library.

@see: U{https://rg3.github.io/youtube-dl/}
"""

from __future__ import unicode_literals
import os
import shutil
import logging
import youtube_dl


class Downloader(object):
    """
    Download mp3 file from YouTube.

    Class download file to cache folder. In case of success files are
    moved to output file. Output and cache folder are describe in
    L{YTSettings<ytrss.core.settings.YTSettings>} object.

    @ivar settings: Setting object
    @type settings: L{YTSettings<ytrss.core.settings.YTSettings>}
    @ivar url: URL to download
    @type url: str
    @ivar output_path: Path to output folder
    @type output_path: str
    """
    def __init__(self, settings, url):
        """
        Downloader constructor.

        @param self: object handler
        @type self: L{Downloader}
        @param settings: settings handler
        @type settings: L{YTSettings<ytrss.core.settings.YTSettings>}
        @param url: URL to YouTube movie
        @type url: str
        """
        self.settings = settings
        self.url = url
        self.output_path = settings.output

    def download(self):
        """
        Download YouTube movie.

        Function download movie, convert to mp3 and move to output file.

        @param self: object handler
        @type self: L{Downloader}
        @return: C{True} if success, C{False} otherwise
        @rtype: Boolean
        """
        status = 0
        cache_path = self.settings.cache_path
        try:
            os.makedirs(cache_path)
        except OSError:
            pass
        current_path = os.getcwd()
        os.chdir(cache_path)

        logging.info("url: %s", self.url)
        try:
            command = ['--extract-audio', '--audio-format', 'mp3',
                       '-o', "%(uploader)s - %(title)s.%(ext)s", self.url]
            youtube_dl.main(command)
        except SystemExit as ex:
            if ex.code is None:
                status = 0
            else:
                status = ex.code

        finded = False
        for find_file in os.listdir(cache_path):
            if find_file.endswith(".mp3"):
                source_path = os.path.join(cache_path, find_file)
                destination_path = os.path.join(self.output_path, find_file)
                logging.debug("source_path: %s", source_path)
                logging.debug("destination_path: %s", destination_path)
                shutil.move(source_path, destination_path)
                finded = True

        os.chdir(current_path)
        return status == 0 and finded



#!/usr/bin/env python
# -*- coding: utf-8 -*-
###########################################################################
#                                                                         #
#  Copyright (C) 2017  Rafal Kobel <rafalkobel@rafyco.pl>                 #
#                                                                         #
#  This program is free software: you can redistribute it and/or modify   #
#  it under the terms of the GNU General Public License as published by   #
#  the Free Software Foundation, either version 3 of the License, or      #
#  (at your option) any later version.                                    #
#                                                                         #
#  This program is distributed in the hope that it will be useful,        #
#  but WITHOUT ANY WARRANTY; without even the implied warranty of         #
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the           #
#  GNU General Public License for more details.                           #
#                                                                         #
#  You should have received a copy of the GNU General Public License      #
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.  #
#                                                                         #
###########################################################################
"""
CommandLine interface for test.

For checking tests call::

    python -m ytrss.tests
"""

# pylint: disable=W0611
from __future__ import unicode_literals
import unittest
from ytrss.tests import TestYTRSSModule
from ytrss.tests.locker_test import TestLocker


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
###########################################################################
#                                                                         #
#  Copyright (C) 2017  Rafal Kobel <rafalkobel@rafyco.pl>                 #
#                                                                         #
#  This program is free software: you can redistribute it and/or modify   #
#  it under the terms of the GNU General Public License as published by   #
#  the Free Software Foundation, either version 3 of the License, or      #
#  (at your option) any later version.                                    #
#                                                                         #
#  This program is distributed in the hope that it will be useful,        #
#  but WITHOUT ANY WARRANTY; without even the implied warranty of         #
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the           #
#  GNU General Public License for more details.                           #
#                                                                         #
#  You should have received a copy of the GNU General Public License      #
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.  #
#                                                                         #
###########################################################################
"""
Locking program module

This modul allow you to blocking program for make more of one instance.

Example of usage::

    locker = Locker('program_name')
    try:
        locker.lock()
    catch LockerError:
        print("Blocked file")
        exit(1)
    # code of our application
    locker.unlock()

@warn: Before out of program method -L{unlock() <Locker.unlock>} should
    be invoke.

"""

import os
import logging
import tempfile


class LockerError(Exception):
    """ Locker exception """
    pass


class Locker(object):
    """
    Class for blocking running program

    @warn: Before out of program method L{unlock() <Locker.unlock>} should
        be invoke.

    @ivar file_path: program's blocking file
    """
    def __init__(self, identify, direcotry=None):
        """
        Locker constructor

        @param self: object handle
        @type self: L{Locker}
        @param identify: identification of blocking application
        @type identify: str
        @param direcotry: path to direcotory with blocking file
            (if direcotry is C{None} file will be in temporary path)
        """
        if direcotry is None:
            tmp = tempfile.gettempdir()
        else:
            tmp = direcotry
        self.file_path = os.path.join(tmp, identify)
        logging.debug("lock path: %s", self.file_path)

    def is_lock(self):
        """
        Is program lock.

        @param self: object handle
        @type self: L{Locker}
        @return: C{True} is blocking on, C{False} otherwise.
        @rtype: Boolean
        """
        return os.path.isfile(self.file_path)

    def lock(self):
        """
        Lock program.

        @param self: object handle
        @type self: L{Locker}
        @raise LockerError: in case of locked
        """
        logging.debug("Lock program: %s", self.file_path)
        if self.is_lock():
            raise LockerError
        open(self.file_path, 'a').close()

    def unlock(self):
        """
        Unlock program.

        @param self: object handle
        @type self: L{Locker}
        @warn: This method not raise any exception.
        """
        logging.debug("Unlock program: %s", self.file_path)
        if self.is_lock():
            os.remove(self.file_path)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
###########################################################################
#                                                                         #
#  Copyright (C) 2017  Rafal Kobel <rafalkobel@rafyco.pl>                 #
#                                                                         #
#  This program is free software: you can redistribute it and/or modify   #
#  it under the terms of the GNU General Public License as published by   #
#  the Free Software Foundation, either version 3 of the License, or      #
#  (at your option) any later version.                                    #
#                                                                         #
#  This program is distributed in the hope that it will be useful,        #
#  but WITHOUT ANY WARRANTY; without even the implied warranty of         #
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the           #
#  GNU General Public License for more details.                           #
#                                                                         #
#  You should have received a copy of the GNU General Public License      #
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.  #
#                                                                         #
###########################################################################
"""
Finding YouTube movie urls.
"""

from __future__ import unicode_literals
import logging
from ytrss.core.ytdown import YTDown


class URLFinder(object):
    """
    Finding YouTube movie urls from configuration.

    @ivar tab: table with source's urls
    @type tab: list
    """
    def __init__(self, settings=None):
        """
        URLFinder constructor.

        @param self: handle object
        @type self: L{URLFinder}
        @param settings: configuration settings
        @type settings: {ytrss.core.settings.YTSettings}
        """
        self.tab = []
        if settings is not None:
            self.add_user_url(settings.urls)
            self.add_playlist_url(settings.playlists)

    def add_user_url(self, url):
        """
        Add subscription code url.

        @param self: handle object
        @type self: L{URLFinder}
        @param url: subscription's code
        @type url: str
        """
        if isinstance(url, list):
            for elem in url:
                self.add_user_url(elem)
        else:
            logging.debug("add user url: %s", url)
            self.tab.append(YTDown(url, link_type="user"))

    def add_playlist_url(self, url):
        """
        Add playlist code url.

        @param self: handle object
        @type self: L{URLFinder}
        @param url: playlist's code
        @type url: str
        """
        if isinstance(url, list):
            for elem in url:
                self.add_playlist_url(elem)
        else:
            logging.debug("add playlist url: %s", url)
            self.tab.append(YTDown(url, link_type="playlist"))

    def get_urls(self):
        """
        Get urls to YouTube movies.

        @param self: handle object
        @type self: L{URLFinder}
        """
        urls = []
        for elem in self.tab:
            logging.debug("Contener: %s", elem)
            addresses = elem.get_urls()
            for address in addresses:
                logging.debug("El: %s", address)
                urls.append(address)
        return urls



#!/usr/bin/env python
# -*- coding: utf-8 -*-
###########################################################################
#                                                                         #
#  Copyright (C) 2017  Rafal Kobel <rafalkobel@rafyco.pl>                 #
#                                                                         #
#  This program is free software: you can redistribute it and/or modify   #
#  it under the terms of the GNU General Public License as published by   #
#  the Free Software Foundation, either version 3 of the License, or      #
#  (at your option) any later version.                                    #
#                                                                         #
#  This program is distributed in the hope that it will be useful,        #
#  but WITHOUT ANY WARRANTY; without even the implied warranty of         #
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the           #
#  GNU General Public License for more details.                           #
#                                                                         #
#  You should have received a copy of the GNU General Public License      #
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.  #
#                                                                         #
###########################################################################
"""
Command line program to checking movie's URL in subscription.

Program checking subscription and playlist from config and save it
to downloading file. It's recomended to add this file to crontab or call
it manually.

Example usage
=============

To invoke program type in your console::

    ytrss_subs

or::

    python -m ytrss.subs

for more option call program with flag C{--help}
"""

from __future__ import unicode_literals
from __future__ import print_function
import logging
from argparse import ArgumentParser
from ytrss import get_version
from ytrss.core import DownloadQueue
from ytrss.core.settings import YTSettings
from ytrss.core.settings import SettingException
from ytrss.core.url_finder import URLFinder
try:
    import argcomplete
except ImportError:
    pass


def __option_args(argv=None):
    """
    Parsing argument for command line program.

    @param argv: Option parameters
    @type argv: list
    @return: parsed arguments
    """
    parser = ArgumentParser(description="Save urls from Youtube's "
                                        "subscription or playlists to file.",
                            prog='ytrss_subs')
    parser.add_argument("-v", "--version", action='version',
                        version='%(prog)s {}'.format(get_version()))
    parser.add_argument("-c", "--conf", dest="configuration",
                        help="configuration file", default="", metavar="FILE")
    parser.add_argument("-l", "--log", dest="logLevel",
                        choices=['DEBUG', 'INFO', 'WARNING',
                                 'ERROR', 'CRITICAL'],
                        help="Set the logging level")

    try:
        argcomplete.autocomplete(parser)
    except NameError:
        pass
    options = parser.parse_args(argv)
    return options


def prepare_urls(settings):
    """
    Prepare urls for downloader.

    @param settings: Settings handle
    @type settings: L{YTSettings<ytrss.core.settings.YTSettings>}
    """
    logging.info("Prepare new urls")
    finder = URLFinder(settings)
    urls = finder.get_urls()
    queue = DownloadQueue(settings)
    for url in urls:
        if queue.queue_mp3(url):
            print("Nowy element: {}".format(url))
        else:
            logging.info("Element istnieje: %s", url)


def main(argv=None):
    """
    Main function for command line program.

    @param argv: Option parameters
    @type argv: list
    """
    options = __option_args(argv)
    logging.basicConfig(format='%(asctime)s - %(name)s '
                               '- %(levelname)s - %(message)s',
                        level=options.logLevel)
    logging.debug("Debug mode: Run")
    try:
        settings = YTSettings(options.configuration)
    except SettingException:
        print("Configuration file not exist.")
        exit(1)
    prepare_urls(settings, options)


if __name__ == "__main__":
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
###########################################################################
#                                                                         #
#  Copyright (C) 2017  Rafal Kobel <rafalkobel@rafyco.pl>                 #
#                                                                         #
#  This program is free software: you can redistribute it and/or modify   #
#  it under the terms of the GNU General Public License as published by   #
#  the Free Software Foundation, either version 3 of the License, or      #
#  (at your option) any later version.                                    #
#                                                                         #
#  This program is distributed in the hope that it will be useful,        #
#  but WITHOUT ANY WARRANTY; without even the implied warranty of         #
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the           #
#  GNU General Public License for more details.                           #
#                                                                         #
#  You should have received a copy of the GNU General Public License      #
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.  #
#                                                                         #
###########################################################################
"""
Tools for downloading mp3 from YouTube subscription and playlists.

Insallation
===========

There are a two metod of installation C{ytrss} module.

From PyPI repository::

    pip install ytrss

From sources::

    git clone git@bitbucket.org:rafyco/ytrss.git
    cd ytrss
    python setup.py install

Usage
=====

Before you using this tools you should create configuration file. More
information you can find L{here<ytrss.core.settings>}.

YTRSS allow you to run a few command-line tool.

    - L{ytrss_daemon<ytrss.daemon>}
    - L{ytdown<ytrss.ytdown>}
    - L{ytrss_subs<ytrss.subs>}

"""

from __future__ import unicode_literals


def get_version():
    """ Get version of ytrss package. """
    return "0.1.5"

__version__ = get_version()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
###########################################################################
#                                                                         #
#  Copyright (C) 2017  Rafal Kobel <rafalkobel@rafyco.pl>                 #
#                                                                         #
#  This program is free software: you can redistribute it and/or modify   #
#  it under the terms of the GNU General Public License as published by   #
#  the Free Software Foundation, either version 3 of the License, or      #
#  (at your option) any later version.                                    #
#                                                                         #
#  This program is distributed in the hope that it will be useful,        #
#  but WITHOUT ANY WARRANTY; without even the implied warranty of         #
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the           #
#  GNU General Public License for more details.                           #
#                                                                         #
#  You should have received a copy of the GNU General Public License      #
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.  #
#                                                                         #
###########################################################################
"""
Testing Locker module

@see: L{Locker<ytrss.core.locker>}
"""

from __future__ import unicode_literals
import os
import unittest
import tempfile
from ytrss.core.locker import Locker
from ytrss.core.locker import LockerError


class TestLocker(unittest.TestCase):
    """
    Locker tests.

    Testing for L{ytrss.core.locker} module
    """
    def setUp(self):
        """ Prepare container for tests. """
        self.lockers = []

    def tearDown(self):
        """ Remove all locks after testing. """
        for lock in self.lockers:
            cleaned_locker = Locker(lock)
            cleaned_locker.unlock()

    def prepare_locker(self, text):
        """ Prepare locker object for testing. """
        self.lockers.append(text)
        return Locker(text)

    def test_usage(self):
        """ Testing usage of module. """
        tested_locker = self.prepare_locker("ytrss_module_test")

        self.assertFalse(tested_locker.is_lock())
        tested_locker.lock()
        self.assertTrue(tested_locker.is_lock())
        tested_locker.unlock()
        self.assertFalse(tested_locker.is_lock())

    def test_double_lock(self):
        """ Testing is double lock raise an error. """
        tested_locker = self.prepare_locker("ytrss_double_test")

        tested_locker.lock()
        with self.assertRaises(LockerError):
            tested_locker.lock()
        tested_locker.unlock()

    def test_double_unlock(self):
        """ Locker should not raise an exception in case of double unlock. """
        tested_locker = self.prepare_locker("ytrss_double_unlock_test")
        self.assertFalse(tested_locker.is_lock())
        tested_locker.lock()
        self.assertTrue(tested_locker.is_lock())
        tested_locker.unlock()
        self.assertFalse(tested_locker.is_lock())
        tested_locker.unlock()
        self.assertFalse(tested_locker.is_lock())

    def test_separation(self):
        """ Testing if locker with different identify are not block each other. """
        tested_locker_1 = self.prepare_locker("ytrss_separation_1_test")
        tested_locker_2 = self.prepare_locker("ytrss_separation_2_test")

        self.assertFalse(tested_locker_1.is_lock())
        self.assertFalse(tested_locker_2.is_lock())

        tested_locker_1.lock()
        self.assertTrue(tested_locker_1.is_lock())
        self.assertFalse(tested_locker_2.is_lock())

    def test_duplicate_instance(self):
        """ Testing if locker with identical identify can block each other. """
        tested_locker_1 = self.prepare_locker("ytrss_duplicate_test")
        tested_locker_2 = self.prepare_locker("ytrss_duplicate_test")
        self.assertNotEqual(tested_locker_1, tested_locker_2)

        self.assertFalse(tested_locker_1.is_lock())
        self.assertFalse(tested_locker_2.is_lock())

        tested_locker_1.lock()

        self.assertTrue(tested_locker_1.is_lock())
        self.assertTrue(tested_locker_2.is_lock())

        tested_locker_2.unlock()

        self.assertFalse(tested_locker_1.is_lock())
        self.assertFalse(tested_locker_2.is_lock())

    def test_path_locker(self):
        """ Testing if locker can be hosted in specific direcotry. """
        directory = os.path.join(tempfile.gettempdir(), "ytrss_test_path")
        try:
            os.makedirs(directory)
        except OSError:
            pass
        test_name = "ytrss_path_test"
        tested_locker = Locker(test_name, directory)
        self.assertEqual(os.path.join(directory, test_name), tested_locker.file_path)
        tested_locker.lock()
        self.assertTrue(os.path.isfile(os.path.join(directory, test_name)))
        tested_locker.unlock()
        self.assertFalse(os.path.isfile(os.path.join(directory, test_name)))
        os.rmdir(directory)

    def test_separation_direcotry(self):
        """
        Testing if locker with directory not blocking another
        locker with the same idefify.
        """
        directory = os.path.join(tempfile.gettempdir(), "ytrss_test_path")
        try:
            os.makedirs(directory)
        except OSError:
            pass
        test_name = "ytrss_path_test"
        tested_locker_d = Locker(test_name, directory)
        tested_locker = self.prepare_locker(test_name)
        tested_locker_d.lock()
        self.assertTrue(tested_locker_d.is_lock())
        self.assertFalse(tested_locker.is_lock())

        tested_locker_d.unlock()
        os.rmdir(directory)


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
###########################################################################
#                                                                         #
#  Copyright (C) 2017  Rafal Kobel <rafalkobel@rafyco.pl>                 #
#                                                                         #
#  This program is free software: you can redistribute it and/or modify   #
#  it under the terms of the GNU General Public License as published by   #
#  the Free Software Foundation, either version 3 of the License, or      #
#  (at your option) any later version.                                    #
#                                                                         #
#  This program is distributed in the hope that it will be useful,        #
#  but WITHOUT ANY WARRANTY; without even the implied warranty of         #
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the           #
#  GNU General Public License for more details.                           #
#                                                                         #
#  You should have received a copy of the GNU General Public License      #
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.  #
#                                                                         #
###########################################################################
"""
Module to download list of YouTube movie ulrs from codes.
"""

from __future__ import unicode_literals
import logging
try:
    from urllib.request import urlopen
except ImportError:
    from urllib import urlopen
from xml.dom import minidom


class YTDown(object):
    """
    Class to download list of YouTube movie urls.

    @ivar code: Code for parsing source
    @type code: str
    @ivar link_type: type of parsing source.
        (I{user} for subscription or I{playlist} for playlist)
    @type link_type: str
    """

    def __init__(self, code, link_type='user'):
        """
        YTDown constructor.

        @param self: object handle
        @type self: L{YTDown}
        @param code: code for parsing source
        @type code: str
        @param link_type: type of source [user|playlist]
        @type link_type: str
        @raise AttributeError: lint_type is not user or playlist
        """
        self.code = code
        self.link_type = link_type
        if self.link_type != "user" and self.link_type != "playlist":
            raise AttributeError("link_type must be 'user' or 'playlist'")

    def __build_url(self):
        """
        Build url to rss source from id save in object.

        @param self: object handle
        @type self: L{YTDown}
        @return: url to rss
        @rtype: str
        """
        patern = "channel_id"
        if self.link_type == 'playlist':
            patern = "playlist_id"
        return "https://www.youtube.com/feeds/videos.xml?{}={}".format(patern, self.code)

    @staticmethod
    def __youtube_list_from_address(address):
        """
        Get movie urls from rss address.

        @param address: address to playlist or subscription rss.
        @type address: list
        @return: list of movie urls
        @rtype: list
        """
        xml_str = urlopen(address).read()
        xmldoc = minidom.parseString(xml_str)
        tags = xmldoc.getElementsByTagName('link')
        result = []
        iterator = 0
        for elem in tags:
            if iterator != 0 and iterator != 1:
                url = elem.getAttribute("href")
                result.append(url)
            iterator = iterator + 1
        return result

    def get_urls(self):
        """
        Get movie urls for object.

        @param self: object handle
        @type self: L{YTDown}
        @return: list of movie urls
        @rtype: list
        """
        url = self.__build_url()
        logging.debug("URL: %s", url)
        return YTDown.__youtube_list_from_address(url)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
###########################################################################
#                                                                         #
#  Copyright (C) 2017  Rafal Kobel <rafalkobel@rafyco.pl>                 #
#                                                                         #
#  This program is free software: you can redistribute it and/or modify   #
#  it under the terms of the GNU General Public License as published by   #
#  the Free Software Foundation, either version 3 of the License, or      #
#  (at your option) any later version.                                    #
#                                                                         #
#  This program is distributed in the hope that it will be useful,        #
#  but WITHOUT ANY WARRANTY; without even the implied warranty of         #
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the           #
#  GNU General Public License for more details.                           #
#                                                                         #
#  You should have received a copy of the GNU General Public License      #
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.  #
#                                                                         #
###########################################################################
"""
Settings parser module.

File are read from the first of the following location.

For MS Windows

    - I{~/YTRSS/config}

For Linux (and other systems)

    - I{/etc/subs_config}
    - I{~/.config/ytrss/config}

In some case you can set file path in constructor or read data in json format.

Config file structure
=====================

Configuration file is in a json format with following option:

    - I{output} - Path to folder where program should download files.
    - I{subscription} - List of subscription.
        For more see: L{parsing function
        <ytrss.core.settings.YTSettings.__parse_subsctiptions>}

Example file
============

code of example file::

    {
        "output"   : "<output_file>",
        "subscriptions" : [
            {
                "code"    : "<playlist_id>",
                "type"    : "playlist"
            },
            {
                "code"    : "<subscritpion_id>"
            },
            {
                "code"    : "<subscription_id>",
                "enabled" : false
            }
        ]
    }

"""

from __future__ import unicode_literals
import os
import sys
import json
import logging


class SettingException(Exception):
    """ Settings exception. """
    pass


class YTSettings(object):
    """
    Parser settings for ytrss.

    Class read settings from parameters or from default conf file.

    @ivar url_rss: file with list of read urls
    @type url_rss: str
    @ivar download_file: file ready to download
    @type download_file: str
    @ivar url_backup: file with actuall downloading files
    @type url_backup: str
    @ivar history_file: file with successfull download urls
    @type history_file: str
    @ivar err_file: file with urls with not downloaded
    @type err_file: str
    @ivar cache_path: path to cache folder
    @type cache_path: str
    """

    urls = []
    """ @cvar: list of subscription codes """
    playlists = []
    """ @cvar: list of playlist codes """

    def __init__(self, conf_file="", conf_str=""):
        """
        YTSettings constructor.

        @param self: object handle
        @type self: L{YTSettings}
        @param conf_file: Path to configuration file
        @type conf_file: str
        @param conf_str: Json format string configuration
        @type conf_str: str
        @raise SettingException: In case of C{conf_file} and C{conf_str}
            are not set.
        """
        conf_path = ""
        if sys.platform.lower().startswith('win'):
            conf_path = os.path.join("~\\YTRSS", conf_path)
        else:
            conf_path = os.path.join("~/.config/ytrss", conf_path)
        self.conf_path = os.path.expanduser(conf_path)

        try:
            os.makedirs(self.conf_path)
        except OSError:
            pass

        data = {}
        if conf_str is not "":
            data = json.load(conf_str)
        else:
            conf_find = self.__check_configuration_file(conf_file)
            logging.debug("Configuration file: %s", conf_find)
            with open(conf_find) as data_file:
                data = json.load(data_file)

        if data is {}:
            raise SettingException

        self.__parse_data(data)

    def __parse_data(self, data):
        """
        Save parsing data from json to object.

        @param self: object handle
        @type self: L{YTSettings}
        @param data: data from json
        @type data: dict
        @see: L{ytrss.core.settings}
        """
        if 'output' in data:
            self.output = os.path.expanduser(data['output'])
        else:
            self.output = os.path.expanduser("~/ytrss_output")
            try:
                os.makedirs(self.output)
            except OSError:
                pass

        self.__parse_subsctiptions(data['subscriptions'])

        self.url_rss = self.__conf_file_path("rss_remember.txt")
        self.download_file = self.__conf_file_path("download_yt.txt")
        self.url_backup = self.__conf_file_path("download_yt_last.txt")
        self.history_file = self.__conf_file_path("download_yt_history.txt")
        self.err_file = self.__conf_file_path("download_yt.txt.err")
        self.cache_path = self.__conf_file_path("cache")

    def __str__(self):
        """
        Display settings information.

        @param self: object handle
        @type self: L{YTSettings}
        @return: formated configuration
        @rtype: str
        """
        result = "Youtube configuration:\n\n"
        result += "output-file: %s\n" % self.output
        for elem in self.urls:
            result += "\turlfile: %s\n" % elem
        for elem in self.playlists:
            result += "\tplaylist: %s\n" % elem
        return result

    def __conf_file_path(self, file_name):
        """
        Create path to file in working folder.

        @param self: object handle
        @type self: L{YTSettings}
        @param file_name: name of searching file
        @type file_name: str
        @return: path to searching file
        @rtype: str
        """
        return os.path.join(self.conf_path, file_name)

    def __check_configuration_file(self, conf=""):
        """
        Get path to configuration file.

        @param self: object handle
        @type self: L{YTSettings}
        @param conf: path to prefer configuration file
        @type conf: str
        @return: path to existing configuration file
        @rtype: str
        @raise SettingException: configuration file not exist
        """
        if conf != "":
            if os.path.expanduser(conf):
                return conf
            else:
                raise SettingException("file: '%s' not exist.")
        conf_file_path = self.__conf_file_path("config")
        if os.path.isfile(conf_file_path):
            return conf_file_path
        elif not(sys.platform.lower().startswith('win')) and os.path.isfile("/etc/subs_config"):
            return "/etc/subs_config"
        else:
            raise SettingException("Cannot find configuration file.")

    def __parse_subsctiptions(self, subs):
        """
        Save parsing information about subscriptions and playlists.

        Information is a list of dict with following fileds.

            - I{type [ url | playlist ]} - type of subscription code. C{url}
                for usual subscription, C{playlist} for playlists.
                (C{url} is default)
            - I{code} - identificator to source of movies. It can be extracted
                from url.
            - I{enabled} - Is element should be downloaded.
                (C{True} is default)


        @param self: object handle
        @type self: L{YTSettings}
        @param subs: list of dicts
        @type subs: list
        """
        for elem in subs:
            if 'type' in elem:
                link_type = elem['type']
            else:
                link_type = 'url'
            if 'enabled' in elem and not elem['enabled']:
                continue
            if link_type == 'playlist':
                self.playlists.append(elem['code'])
            else:
                self.urls.append(elem['code'])



#!/usr/bin/env python
# -*- coding: utf-8 -*-
###########################################################################
#                                                                         #
#  Copyright (C) 2017  Rafal Kobel <rafalkobel@rafyco.pl>                 #
#                                                                         #
#  This program is free software: you can redistribute it and/or modify   #
#  it under the terms of the GNU General Public License as published by   #
#  the Free Software Foundation, either version 3 of the License, or      #
#  (at your option) any later version.                                    #
#                                                                         #
#  This program is distributed in the hope that it will be useful,        #
#  but WITHOUT ANY WARRANTY; without even the implied warranty of         #
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the           #
#  GNU General Public License for more details.                           #
#                                                                         #
#  You should have received a copy of the GNU General Public License      #
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.  #
#                                                                         #
###########################################################################
"""
Module with additional class for ytrss tools.
"""

from __future__ import unicode_literals
from __future__ import print_function
import os
import abc
import json
import logging


class URLRemembererError(Exception):
    """ URLRememberer Exception. """
    pass


class UrlRememberer(object):
    """
    Remember URLs or read it from file.

    @ivar file_name: name of checking file
    @type file_name: str
    @ivar file_data: source of file with data
    @type file_data: str
    @ivar database: list of checking urls
    @type database: list
    """
    def __init__(self, file_name):
        """
        UrlRememberer constructor.

        @param self: object handle
        @type self: L{UrlRememberer}
        @param file_name: name of checking file
        @type file_name: str
        """
        self.file_data = ""
        self.database = []
        self.file_name = ""
        logging.debug("url_remember: %s", file_name)
        self.file_name = file_name
        try:
            plik = open(self.file_name)
            try:
                self.file_data = plik.read()
            finally:
                plik.close()
            for elem in self.file_data.split('\n'):
                if elem != "":
                    self.database.append(elem)

        except IOError as ex:
            logging.debug("Unknown error %s", ex)

    def add_element(self, address):
        """
        Add address to base.

        @param self: object handle
        @type self: L{UrlRememberer}
        @param address: adding address
        @type address: str
        """
        if address == "":
            return
        self.database.append(address)
        if self.file_name == "":
            return
        plik = open(self.file_name, 'a')
        plik.writelines(address + '\n')
        plik.close()
        self.file_data = "{}\n{}\n".format(self.file_data, address)

    def is_new(self, address):
        """
        Check is URL not exist in file.

        @param self: object handle
        @type self: L{UrlRememberer}
        @param address: address to check
        @type address: str
        @return: C{True} if address new, C{False} otherwise
        @rtype: Boolean
        """
        logging.debug("Sprawdzanie pliku: %s", self.file_name)
        for elem in self.database:
            logging.debug("Analiza: %s", elem)
            if elem == address:
                logging.debug("old element %s", address)
                return False
        return True

    def save_as(self, file_name):
        """
        Save copy to file.

        @param self: object handle
        @type self: L{UrlRememberer}
        @param file_name: path to save
        @type file_name: str
        """
        plik = open(file_name, 'a')
        for elem in self.database:
            plik.writelines(elem + '\n')
        plik.close()

    def delete_file(self):
        """
        Delete file with data.

        @param self: object handle
        @type self: L{UrlRememberer}
        """
        if self.file_name == "":
            return
        os.remove(self.file_name)
        self.file_name = ""
        self.file_data = ""

    def read_backup(self, backup_file):
        """
        Merge with another file.

        @param self: object handle
        @type self: L{UrlRememberer}
        @param backup_file: path to read data
        @type backup_file: str
        @warn: If file not exist method not raise any exception
        """
        logging.debug("read backup: %s", backup_file)
        if os.path.isfile(backup_file):
            try:
                plik = open(backup_file)
                try:
                    file_data = plik.read()
                finally:
                    plik.close()
                for elem in file_data.split('\n'):
                    if elem != "":
                        self.add_element(elem)
            except IOError:
                pass
            os.remove(backup_file)


class DownloadQueue(object):
    """
    Class saving urls to download.

    @ivar url_rss: path to file with readed urls
    @type url_rss: str
    @ivar download_file: file ready to download
    @type download_file: str
    @ivar rememberer: object remebering urls
    @type rememberer: L{UrlRememberer}
    """

    def __init__(self, settings, base_file=None):
        """
        DownloadQueue constructor.

        @param self: object handle
        @type self: L{DownloadQueue}
        @param settings: settings for DownloadQueue
        @type settings: L{YTSettings<ytrss.core.settings.YTSettings>}
        @param base_file: path to file with remember subscription
        @type base_file: str

        @note:: if C{base_file} not set, it is get from settings object.
        """
        self.url_rss = settings.url_rss
        self.download_file = settings.download_file
        self.download_yt = UrlRememberer(self.download_file)
        if base_file is None:
            base_file = settings.url_rss
        logging.debug(base_file)
        self.rememberer = UrlRememberer(base_file)

    def queue_mp3(self, address):
        """
        Add address to download.

        @param self: object handle
        @type self: L{DownloadQueue}
        @param address: adress to download
        @type address: str

        @return: C{True} if C{address} can be downloaded, C{False} otherwise
        @rtype: Boolean
        """
        logging.debug("DOWNLOAD: %s", address)
        if self.rememberer.is_new(address):
            logging.debug("Download adress: %s", address)
            self.download_yt.add_element(address)
            self.rememberer.add_element(address)
            return True
        return False



#!/usr/bin/env python
# -*- coding: utf-8 -*-
###########################################################################
#                                                                         #
#  Copyright (C) 2017  Rafal Kobel <rafalkobel@rafyco.pl>                 #
#                                                                         #
#  This program is free software: you can redistribute it and/or modify   #
#  it under the terms of the GNU General Public License as published by   #
#  the Free Software Foundation, either version 3 of the License, or      #
#  (at your option) any later version.                                    #
#                                                                         #
#  This program is distributed in the hope that it will be useful,        #
#  but WITHOUT ANY WARRANTY; without even the implied warranty of         #
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the           #
#  GNU General Public License for more details.                           #
#                                                                         #
#  You should have received a copy of the GNU General Public License      #
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.  #
#                                                                         #
###########################################################################
"""
Add one or more URL address to downlad file.

Example usage
=============

To invoke program type in your console::

    ytdown [url]

or::

    python -m ytrss.ytdown [url]

for more option call program with flag C{--help}
"""

from __future__ import unicode_literals
from __future__ import print_function
import sys
import time
import logging
from argparse import ArgumentParser
from ytrss import get_version
from ytrss.daemon import download_all_movie
from ytrss.subs import prepare_urls
from ytrss.core import DownloadQueue
from ytrss.core.settings import YTSettings
from ytrss.core.settings import SettingException
from daemonocle import Daemon
from daemonocle.exceptions import DaemonError
try:
    import argcomplete
except ImportError:
    pass


def __option_args(argv=None):
    """
    Parsing argument for command line program.

    @param argv: Option parameters
    @type argv: list
    @return: parsed arguments
    """
    parser = ArgumentParser(description="Save one or more urls from "
                                        "Youtube to file.",
                            prog='ytdown')
    parser.add_argument("-v", "--version", action='version',
                        version='%(prog)s {}'.format(get_version()))
    parser.add_argument("-c", "--conf", dest="configuration",
                        help="configuration file", default="", metavar="FILE")
    parser.add_argument("-l", "--log", dest="logLevel",
                        choices=['DEBUG', 'INFO', 'WARNING',
                                 'ERROR', 'CRITICAL'],
                        help="Set the logging level")
    parser.add_argument("-s", "--show", action="store_true",
                        dest="show_config", default=False,
                        help="Write configuration")
    parser.add_argument("-r", "--read", action="store_true",
                        dest="daemon_run", default=False,
                        help="Read urls to download from rss")
    parser.add_argument("-d", "--download", action="store_true",
                        dest="download_run", default=False,
                        help="Download all movies to output path")
    parser.add_argument("urls", nargs='*', default=[], type=str,
                        help="Url to download.")
    try:
        argcomplete.autocomplete(parser)
    except NameError:
        pass
    return parser.parse_args(argv)


def main(argv=None):
    """
    Main function for command line program.

    @param argv: Option parameters
    @type argv: list
    """
    options = __option_args(argv)
    logging.basicConfig(format='%(asctime)s - %(name)s - '
                               '%(levelname)s - %(message)s',
                        level=options.logLevel)
    try:
        settings = YTSettings(options.configuration)
    except SettingException:
        print("Configuration file not exist.")
        exit(1)

    if options.show_config:
        print(settings)
        exit()

    if options.daemon_run or options.download_run:
        prepare_urls(settings)

    if options.download_run:
        download_all_movie(settings)

    if (len(options.urls) < 1 and not options.download_run and
            not options.daemon_run):
        print("Require url to download")
        exit(1)

    queue = DownloadQueue(settings)
    for url in options.urls:
        if queue.queue_mp3(url):
            print("Filmik zostanie pobrany: {}".format(url))
        else:
            print("Filmik nie zostanie pobrany: {}".format(url))


def daemon_main():
    """
    Daemon main function.
    """
    while True:
        try:
            settings = YTSettings()
            try:
                prepare_urls(settings)
            except SystemExit:
                pass
            try:
                download_all_movie(settings)
            except SystemExit:
                pass
        except SettingException:
            logging.error("Configuration file not exist.")
        except Exception as ex:
            logging.error("Unknown error: {}".format(ex))
        # Wait 10 min.
        time.sleep(60 * 10)


def daemon_error_print():
    """
    Print error message in case of invalid argument.
    """
    print("[info] Usage: /etc/init.d/ytrss {start|stop|restart|status}")


def daemon(argv=None):
    """
    Daemon script function.

    This script turn on daemon.

    @param argv: Option parameters
    @type argv: list
    """
    daemon = Daemon(worker=daemon_main,
                    pidfile='/var/run/daemonocle_example.pid'
                    )
    if argv is None:
        argv = sys.argv
    logging.basicConfig(
        filename='/var/log/ytrss_daemon.log',
        level=logging.INFO, format='%(asctime)s [%(levelname)s] %(message)s',
    )
    if len(argv) != 2:
        daemon_error_print()
        exit(1)
    try:
        if argv[1] == "start":
            try:
                YTSettings()
            except SettingException:
                print("Configuration file not exist.")
                exit(1)
        daemon.do_action(argv[1])
        sys.exit(0)
    except (DaemonError, IndexError):
        daemon_error_print()
        sys.exit(1)


if __name__ == "__main__":
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
###########################################################################
#                                                                         #
#  Copyright (C) 2017  Rafal Kobel <rafalkobel@rafyco.pl>                 #
#                                                                         #
#  This program is free software: you can redistribute it and/or modify   #
#  it under the terms of the GNU General Public License as published by   #
#  the Free Software Foundation, either version 3 of the License, or      #
#  (at your option) any later version.                                    #
#                                                                         #
#  This program is distributed in the hope that it will be useful,        #
#  but WITHOUT ANY WARRANTY; without even the implied warranty of         #
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the           #
#  GNU General Public License for more details.                           #
#                                                                         #
#  You should have received a copy of the GNU General Public License      #
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.  #
#                                                                         #
###########################################################################

from __future__ import unicode_literals
import os
import codecs
from setuptools import setup
from setuptools import find_packages


def read_markdown(fname):
    fpath = os.path.join(os.path.dirname(__file__), fname)
    try:
        import pypandoc
        return pypandoc.convert(fpath, 'rst')
    except(IOError, ImportError, RuntimeError):
        return codecs.open(fpath).read().decode('utf-8')


def read_description(module_name):
    module_doc = __import__(module_name).__doc__.splitlines()
    result = ""
    for line in module_doc:
        if line:
            result = line
            break
    return result

version = __import__('ytrss').get_version()

console_scripts = [
            'ytdown = ytrss.ytdown:main',
            '/etc/init.d/ytrss = ytrss.ytdown:daemon'
        ]

setup(
    name='ytrss',
    version=version,
    license='GNU',
    author="Rafal Kobel",
    author_email="rafalkobel@rafyco.pl",
    description=read_description('ytrss'),
    long_description=read_markdown("README.md"),
    url="http://bitbucket.org/rafyco/ytrss",
    packages=find_packages(),
    include_package_data=True,
    package_dir={'ytrss': 'ytrss'},
    test_suite='ytrss.tests.__main__',
    classifiers=[
        'Environment :: Console',
        'Development Status :: 3 - Alpha',
        'Topic :: Multimedia :: Sound/Audio',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.4',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)'
    ],
    install_requires=[
        'youtube_dl',
        'daemonocle',
        'pep8'
    ],
    entry_points={
        'console_scripts': console_scripts
    },
    platforms='Any',
    keywords='youtube, console, download, rss, mp3, service'
)
